@@ -378,7 +378,7 @@ SV *p5_err_sv(PerlInterpreter *my_perl) {
     return sv_mortalcopy(ERRSV);
 }
 
-SV *p5_call_package_method(PerlInterpreter *my_perl, char *package, char *name, int len, SV *args[], I32 *count, I32*err) {
+SV *p5_call_package_method(PerlInterpreter *my_perl, char *package, char *name, int len, SV *args[], I32 *count, I32 *err, I32 *type) {
     PERL_SET_CONTEXT(my_perl);
     {
         dSP;
@@ -408,6 +408,7 @@ SV *p5_call_package_method(PerlInterpreter *my_perl, char *package, char *name,
         if (*count == 1) {
             retval = POPs;
             SvREFCNT_inc(retval);
+            *type = p5_get_type(my_perl, retval);
         }
         else {
             if (*count > 1) {
@@ -432,7 +433,7 @@ SV *p5_call_package_method(PerlInterpreter *my_perl, char *package, char *name,
     }
 }
 
-SV *p5_call_method(PerlInterpreter *my_perl, char *package, SV *obj, I32 context, char *name, int len, SV *args[], I32 *count, I32 *err) {
+SV *p5_call_method(PerlInterpreter *my_perl, char *package, SV *obj, I32 context, char *name, int len, SV *args[], I32 *count, I32 *err, I32 *type) {
     PERL_SET_CONTEXT(my_perl);
     {
         dSP;
@@ -449,9 +450,14 @@ SV *p5_call_method(PerlInterpreter *my_perl, char *package, SV *obj, I32 context
         if (gv && isGV(gv)) {
             PUSHMARK(SP);
 
-            for (i = 0; i < len; i++) {
-                XPUSHs(sv_2mortal(args[i]));
+            if (len > 1) {
+                XPUSHs(package != NULL ? sv_2mortal(args[0]) : args[0]);
+                for (i = 1; i < len; i++) {
+                    XPUSHs(sv_2mortal(args[i]));
+                }
             }
+            else if (len > 0)
+                XPUSHs(package != NULL ? sv_2mortal((SV*)args) : (SV*)args);
 
             PUTBACK;
 
@@ -466,6 +472,7 @@ SV *p5_call_method(PerlInterpreter *my_perl, char *package, SV *obj, I32 context
             if (*count == 1) {
                 retval = POPs;
                 SvREFCNT_inc(retval);
+                *type = p5_get_type(my_perl, retval);
             }
             else {
                 if (*count > 1) {
@@ -493,7 +500,7 @@ SV *p5_call_method(PerlInterpreter *my_perl, char *package, SV *obj, I32 context
     }
 }
 
-SV *p5_call_function(PerlInterpreter *my_perl, char *name, int len, SV *args[], I32 *count, I32 *err) {
+SV *p5_call_function(PerlInterpreter *my_perl, char *name, int len, SV *args[], I32 *count, I32 *err, I32 *type) {
     PERL_SET_CONTEXT(my_perl);
     {
         dSP;
@@ -522,6 +529,7 @@ SV *p5_call_function(PerlInterpreter *my_perl, char *name, int len, SV *args[],
         if (*count == 1) {
             retval = POPs;
             SvREFCNT_inc(retval);
+            *type = p5_get_type(my_perl, retval);
         }
         else if (*count > 1) {
             retval = (SV *)newAV();
@@ -544,7 +552,7 @@ SV *p5_call_function(PerlInterpreter *my_perl, char *name, int len, SV *args[],
     }
 }
 
-SV *p5_call_code_ref(PerlInterpreter *my_perl, SV *code_ref, int len, SV *args[], I32 *count, I32 *err) {
+SV *p5_call_code_ref(PerlInterpreter *my_perl, SV *code_ref, int len, SV *args[], I32 *count, I32 *err, I32 *type) {
     PERL_SET_CONTEXT(my_perl);
     {
         dSP;
@@ -573,6 +581,7 @@ SV *p5_call_code_ref(PerlInterpreter *my_perl, SV *code_ref, int len, SV *args[]
         if (*count == 1) {
             retval = POPs;
             SvREFCNT_inc(retval);
+            *type = p5_get_type(my_perl, retval);
         } else if (*count > 1) {
             retval = (SV *)newAV();
             av_extend((AV *)retval, *count - 1);
